#!/usr/bin/env bash

listinfo=list_of_oku.edu.mie-u.ac.jp.txt

: main && {

    if [ ! -f "${listinfo}" ] ; then
        exit
    fi

    Otopurl='https*://oku.edu.mie-u.ac.jp'
    Ntopurl='https://okumuralab.org/'

    tmp1=$(mktemp)
    tmp2=$(mktemp)
    trap 'rm -f ${tmp1} ${tmp2} ; exit ' 1 2 3 15

    cat ${listinfo} |
        perl -Mutf8 -CSD -npe '
        # ref(http...,...) タイプ
        s/ref[(](http[^,]+),/\n$1\n/g;

        # URL を抜き出す
        # http:// か https:// で始まり [](){}"と空白を含まない文字列
        # http の h が &#x68; とUNICODE HEX 表現されているものあり
        # decode モジュールを使ってもいいが一箇所だけなので h に決め打ち
        #  11111111  22222222222222222222222
        s/(&#x68;|h)(ttp[^][(){}"[:space:]]+)/\nh$2\n/g;

        ' |
        (
            perl -Mutf8 -CSD -npe 's/^(http:)/ $1/;' |
            rg "https*://oku.edu.mie-u.ac.jp" | LANG=C sort -t/ -k 3 -k 1 | uniq
        ) |
        while read ooo ; do
            nnn=$(
                echo ${ooo} |
                    perl -Mutf8 -CSD -npe 's%^'${Otopurl}'%'${Ntopurl}'%;'
               )
            urls=$(echo $ooo $nnn | tr ' ' '\n' | sort -u)
            for url in ${urls} ; do
                curl --head -silent ${url} > ${tmp2}

                rg -q -e '403 Forbidden' -e '404 Not Found' ${tmp2}
                if [ $? -eq 0 ] ; then
                    check='X'
                else
                    check='-'
                fi

                echo "- [${check}] ${url}"
                cat ${tmp2} |
                    perl -Mutf8 -CSD -npe 's/^([^[:space:]])/      $1/;' |
                    nkf -Lu
            done > ${tmp1}

            tag=$(cat ${tmp1} |
                      awk '$1 ~ /HTTP/{ $1 = ""; print ":", $0}' |
                      xargs
               )
            status=$(echo ${tag} | tr -cd '0-9 ' | xargs | perl -pne 's/ +/→/g;')
            echo "${tag}" | rg -q ': 301 Moved Permanently : 200 OK'
            if [ $? -eq 0 ] ; then
                todo='DONE'
            else
                todo='TODO'
            fi

            echo "** ${todo} ${status} ${ooo}" ${tag}
            cat ${tmp1}
        done

    rm -f ${tmp1} ${tmp2}
}
